// linear-control-pitch-encoder.c : sample for the Balancer robot

// This file is part of ArtLPC. 
// This file is placed into the public domain.

/****************************************************************/
#define NSTATES   4   // internal states of the linear system
#define NOUTPUTS  2   // observable sensor outputs from the linear system
#define NCONTROLS 1   // number of control inputs to the linear system


// gain matrix for the LQR controller
static float lqrNegK[NCONTROLS*NSTATES] = {
    0.0000,  1.7059,  8.6731,  2.4858
};


/****************************************************************/

// State estimate for the linear controller.  This represents the best
// guess as to the actual system state, and is used for control.
static float modelXe[ NSTATES ];

// Control vector for the linear controller.  This is the control
// computed to be applied to the actual system.
static float modelU[ NCONTROLS ];




/****************************************************************/
// Recompute the control output based on the estimated state.
void update_LQR_control(void)
{
  // u = -K * xe;
  matrix_zero( modelU, NCONTROLS );
  matrix_multiply_sum( lqrNegK, modelXe, modelU, NCONTROLS, NSTATES, 1 );
}
